//=======================================================================
//                         USB Target CDC sample
//
// File:
//   usbs_base.c
//
// Abstract:
//   OpenCores USB IP Core control functions
//
//  Created:
//    27 April 2015
//======================================================================
//
// Copyright (c) 2015, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id: d3534360d7ab5d86113439003f13373e5532415e $

//#define INT_MODE

#include <system.h>
#include <altera_avalon_pio_regs.h>
#ifdef INT_MODE
#include <sys/alt_irq.h>
#endif
#include "usbs_base.h"

// device state
st_usbs_env g_usbs_env;

unsigned char g_set_line_coding_buf[] = {
  0x80,0x25,0x00,0x00,0x00,0x00,0x08
};

void set7SEG(char n) {
   //            bit pos
   //     -        0
   //    | |      5 1
   //     -        6
   //    | |      4 2
   //     -        3
  switch (n) {
  case '0':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x3f);
    break;
  case '1':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x06);
    break;
  case '2':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x5b);
    break;
  case '3':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x4f);
    break;
  case '4':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x66);
    break;
  case '5':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x6d);
    break;
  case '6':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x7d);
    break;
  case '7':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x27);
    break;
  case '8':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x7f);
    break;
  case '9':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x67);
    break;
  case 'A':
  case 'a':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x77);
    break;
  case 'B':
  case 'b':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x7c);
    break;
  case 'C':
  case 'c':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x58);
    break;
  case 'D':
  case 'd':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x5e);
    break;
  case 'E':
  case 'e':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x79);
    break;
  case 'F':
  case 'f':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x71);
    break;
  case 'G':
  case 'g':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x3d);
    break;
  case 'H':
  case 'h':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x74);
    break;
  case 'I':
  case 'i':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x04);
    break;
  case 'J':
  case 'j':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x1e);
    break;
  case 'K':
  case 'k':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x75);
    break;
  case 'L':
  case 'l':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x38);
    break;
  case 'M':
  case 'm':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x37);
    break;
  case 'N':
  case 'n':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x54);
    break;
  case 'O':
  case 'o':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x5c);
    break;
  case 'P':
  case 'p':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x73);
    break;
  case 'Q':
  case 'q':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x7b);
    break;
  case 'R':
  case 'r':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x50);
    break;
  case 'S':
  case 's':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x6c);
    break;
  case 'T':
  case 't':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x78);
    break;
  case 'U':
  case 'u':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x1c);
    break;
  case 'V':
  case 'v':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x3e);
    break;
  case 'W':
  case 'w':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x7e);
    break;
  case 'X':
  case 'x':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x76);
    break;
  case 'Y':
  case 'y':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x6e);
    break;
  case 'Z':
  case 'z':
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x1b);
    break;
  default:
    IOWR_ALTERA_AVALON_PIO_DATA(PIO_2_BASE,~0x80);
    break;
  }
}

void endpoint2_trans() {
  g_usbs_env.EP2.active = 0;
  update_pid_ep(2);
  // BULK-IN transaction
  // for next OUT data
  set_control_reg_ep(1);
}

void set_control_reg_ep(int en) {
  switch (en) {
  case 1:
    g_usbs_env.EP1.active = 1;
    USB_ENDPOINT1_CONTROL_REG = 0x3 | (g_usbs_env.EP1.out_data_type << 2);
    break;
  case 2:
    g_usbs_env.EP2.active = 1;
    USB_ENDPOINT2_CONTROL_REG = 0x3 | (g_usbs_env.EP2.out_data_type << 2);
    break;
  case 3:
    g_usbs_env.EP3.active = 1;
    USB_ENDPOINT3_CONTROL_REG = 0x3 | (g_usbs_env.EP3.out_data_type << 2);
    break;
  }
}

void set_control_reg_stall_ep(int en) {
  switch (en) {
  case 1:
    g_usbs_env.EP1.active = 1;
    USB_ENDPOINT1_CONTROL_REG = 0xb;
    break;
  case 2:
    g_usbs_env.EP2.active = 1;
    USB_ENDPOINT2_CONTROL_REG = 0xb;
    break;
  case 3:
    g_usbs_env.EP3.active = 1;
    USB_ENDPOINT3_CONTROL_REG = 0xb;
    break;
  }
}

void update_pid_ep(int en) {
  switch (en) {
  case 1:
    if (g_usbs_env.EP1.out_data_type == 0) g_usbs_env.EP1.out_data_type = 1;
    else g_usbs_env.EP1.out_data_type = 0;
    break;
  case 2:
    if (g_usbs_env.EP2.out_data_type == 0) g_usbs_env.EP2.out_data_type = 1;
    else g_usbs_env.EP2.out_data_type = 0;
    break;
  case 3:
    if (g_usbs_env.EP3.out_data_type == 0) g_usbs_env.EP3.out_data_type = 1;
    else g_usbs_env.EP3.out_data_type = 0;
    break;
  }
}

void set_control_reg_pid0_ep(int en) {
  switch (en) {
  case 1:
    g_usbs_env.EP1.active = 1;
    USB_ENDPOINT1_CONTROL_REG = 0x3;
    break;
  case 2:
    g_usbs_env.EP2.active = 1;
    USB_ENDPOINT2_CONTROL_REG = 0x3;
    break;
  case 3:
    g_usbs_env.EP3.active = 1;
    USB_ENDPOINT3_CONTROL_REG = 0x3;
    break;
  }
}

void set_control_reg_pid1_ep(int en) {
  switch (en) {
  case 1:
    g_usbs_env.EP1.active = 1;
    USB_ENDPOINT1_CONTROL_REG = 0x3|0x4;
    break;
  case 2:
    g_usbs_env.EP2.active = 1;
    USB_ENDPOINT2_CONTROL_REG = 0x3|0x4;
    break;
  case 3:
    g_usbs_env.EP3.active = 1;
    USB_ENDPOINT3_CONTROL_REG = 0x3|0x4;
    break;
  }
}

void endpoint1_trans() {
   // BULK-OUT transaction
   unsigned char buf[64];
   int len = 0;
   int i;
   g_usbs_env.EP1.active = 0;
   update_pid_ep(1);
   len = rx_fifo_cnt(1);
   for (i=0;i<len;i++)
     buf[i] = USB_EP1_RX_FIFO_DATA;
   if (buf[0] == MESSAGE_KEY)
     disp_terminal_message();
   else {
     // EP2 setup
     for (i=0;i<len;i++)
       USB_EP2_TX_FIFO_DATA = buf[i];
     set_control_reg_ep(2);
     set7SEG(buf[0]);
   }
}

void endpoint3_trans() {
}

void disp_terminal_message() {
  // EP2 setup
  USB_EP2_TX_FIFO_CONTROL_REG = 1;
  USB_EP2_TX_FIFO_DATA = 'O';  // IN
  USB_EP2_TX_FIFO_DATA = 'p';  // IN
  USB_EP2_TX_FIFO_DATA = 'e';  // IN
  USB_EP2_TX_FIFO_DATA = 'n';  // IN
  USB_EP2_TX_FIFO_DATA = 'C';  // IN
  USB_EP2_TX_FIFO_DATA = 'o';  // IN
  USB_EP2_TX_FIFO_DATA = 'r';  // IN
  USB_EP2_TX_FIFO_DATA = 'e';  // IN
  USB_EP2_TX_FIFO_DATA = 's';  // IN
  USB_EP2_TX_FIFO_DATA = ' ';  // IN
  USB_EP2_TX_FIFO_DATA = 'U';  // IN
  USB_EP2_TX_FIFO_DATA = 'S';  // IN
  USB_EP2_TX_FIFO_DATA = 'B';  // IN
  USB_EP2_TX_FIFO_DATA = ' ';  // IN
  USB_EP2_TX_FIFO_DATA = 'I';  // IN
  USB_EP2_TX_FIFO_DATA = 'P';  // IN
  USB_EP2_TX_FIFO_DATA = ' ';  // IN
  USB_EP2_TX_FIFO_DATA = 'C';  // IN
  USB_EP2_TX_FIFO_DATA = 'o';  // IN
  USB_EP2_TX_FIFO_DATA = 'r';  // IN
  USB_EP2_TX_FIFO_DATA = 'e';  // IN
  USB_EP2_TX_FIFO_DATA = ' ';  // IN
  USB_EP2_TX_FIFO_DATA = 'C';  // IN
  USB_EP2_TX_FIFO_DATA = 'D';  // IN
  USB_EP2_TX_FIFO_DATA = 'C';  // IN
  USB_EP2_TX_FIFO_DATA = ' ';  // IN
  USB_EP2_TX_FIFO_DATA = 'A';  // IN
  USB_EP2_TX_FIFO_DATA = 'C';  // IN
  USB_EP2_TX_FIFO_DATA = 'M';  // IN
  USB_EP2_TX_FIFO_DATA = ' ';  // IN
  USB_EP2_TX_FIFO_DATA = 'S';  // IN
  USB_EP2_TX_FIFO_DATA = 'a';  // IN
  USB_EP2_TX_FIFO_DATA = 'm';  // IN
  USB_EP2_TX_FIFO_DATA = 'p';  // IN
  USB_EP2_TX_FIFO_DATA = 'l';  // IN
  USB_EP2_TX_FIFO_DATA = 'e';  // IN
  USB_EP2_TX_FIFO_DATA = '.';  // IN
  USB_EP2_TX_FIFO_DATA = '(';  // IN
  USB_EP2_TX_FIFO_DATA = 'L';  // IN
  USB_EP2_TX_FIFO_DATA = 'o';  // IN
  USB_EP2_TX_FIFO_DATA = 'g';  // IN
  USB_EP2_TX_FIFO_DATA = 'i';  // IN
  USB_EP2_TX_FIFO_DATA = 'c';  // IN
  USB_EP2_TX_FIFO_DATA = '+';  // IN
  USB_EP2_TX_FIFO_DATA = 'C';  // IN
  USB_EP2_TX_FIFO_DATA = 'P';  // IN
  USB_EP2_TX_FIFO_DATA = 'U';  // IN
  USB_EP2_TX_FIFO_DATA = ')';  // IN
  USB_EP2_TX_FIFO_DATA = '\r';  // IN
  USB_EP2_TX_FIFO_DATA = '\n';  // IN
  set_control_reg_ep(2);
}

void reset_event_proc() {
  //printf("RESET_EVENT\n");
  g_usbs_env.usb_device_state = UDS_DEFAULT;
  g_usbs_env.EP1.active = 0;
  g_usbs_env.EP1.out_data_type = 0;
  g_usbs_env.EP2.active = 0;
  g_usbs_env.EP2.out_data_type = 0;
  g_usbs_env.EP3.active = 0;
  g_usbs_env.EP3.out_data_type = 0;

  USB_EP1_RX_FIFO_CONTROL_REG = 1;
  USB_EP1_TX_FIFO_CONTROL_REG = 1;
  USB_EP2_RX_FIFO_CONTROL_REG = 1;
  USB_EP2_TX_FIFO_CONTROL_REG = 1;
  USB_EP3_RX_FIFO_CONTROL_REG = 1;
  USB_EP3_TX_FIFO_CONTROL_REG = 1;
  USB_ENDPOINT1_CONTROL_REG = 0x0;  // not enable & ready
  USB_ENDPOINT2_CONTROL_REG = 0x0;  // not enable & ready
  USB_ENDPOINT3_CONTROL_REG = 0x0;  // not enable & ready
}

int check_ready_bit_ep(int en) {
  switch (en) {
  case 1:
    return (g_usbs_env.EP1.active & !(USB_ENDPOINT1_CONTROL_REG & 0x02));
  case 2:
    return (g_usbs_env.EP2.active & !(USB_ENDPOINT2_CONTROL_REG & 0x02));
  case 3:
    return (g_usbs_env.EP3.active & !(USB_ENDPOINT3_CONTROL_REG & 0x02));
  default:
    return 0;
  }
}

void trans_done_event_proc() {
   if (check_ready_bit_ep(1))  endpoint1_trans();
   if (check_ready_bit_ep(2))  endpoint2_trans();
}

int rx_fifo_cnt(int ep) {
  if (ep == 0) return (USB_EP0_RX_FIFO_DATA_COUNT_MSB << 8) | USB_EP0_RX_FIFO_DATA_COUNT_LSB;
  else
  if (ep == 1) return (USB_EP1_RX_FIFO_DATA_COUNT_MSB << 8) | USB_EP1_RX_FIFO_DATA_COUNT_LSB;
  else
  if (ep == 2) return (USB_EP2_RX_FIFO_DATA_COUNT_MSB << 8) | USB_EP2_RX_FIFO_DATA_COUNT_LSB;
  else
  if (ep == 3) return (USB_EP3_RX_FIFO_DATA_COUNT_MSB << 8) | USB_EP3_RX_FIFO_DATA_COUNT_LSB;
  else return -1;
}

#ifdef INT_MODE
volatile int int_configured;
static void handle_cdc_interrupts(void* context){
  int_configured = USB_CDC_STATUS_REG; // clear int flag
  int_configured = 1;
}

void init_cdc_interrupt() {
  int* ptr;
  alt_ic_isr_register(FM_USBS_WRAPPER_0_IRQ_INTERRUPT_CONTROLLER_ID, FM_USBS_WRAPPER_0_IRQ ,handle_cdc_interrupts, ptr, 0x0); //Register Interrupt
  USB_CDC_INT_MASK_REG = 0; // clear int mask
  int_configured = 0;
}
#endif

void wait_configured() {
  int stat = 0;
#ifdef INT_MODE
  // set int test
  while (!int_configured)
	stat = USB_CDC_INT_MASK_REG;  // dummy read
  int_configured = 0;
#else
  while (!stat)
    stat = USB_CDC_STATUS_REG;
#endif
  USB_ENDPOINT1_CONTROL_REG = 0x1;
  USB_ENDPOINT2_CONTROL_REG = 0x1;
  USB_ENDPOINT3_CONTROL_REG = 0x1;
  set_control_reg_ep(1);
}

void usbs_init() {
  reset_event_proc();
#ifdef INT_MODE
  init_cdc_interrupt();
#endif
}

