//=======================================================================
//                         USB Target CDC sample
//
// File:
//   usbs_base.h
//
// Abstract:
//   OpenCores USB IP Core control functions header
//
//  Created:
//    27 April 2015
//======================================================================
//
// Copyright (c) 2015, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id$

#ifndef USBS_BASE_H_
#define USBS_BASE_H_
#include "usbs_hw_reg.h"
#include "usbs_struct.h"

#define MESSAGE_KEY  0x08  // CTRL-H

void usbs_init();

void control_transfer_setup_trans();
void control_transfer_in_trans();
void control_transfer_out_trans();
void show_status();
void endpoint1_trans();
void endpoint2_trans();

void reset_event_proc();
void trans_done_event_proc();
void nak_sent_event_proc();

void set_control_stage(en_usbs_control_transfer_stage s);
int rx_fifo_cnt(int ep);
void disp_terminal_message();
void init_control_cnt(int blen);
int check_ready_bit_ep(int en);
void set_control_reg_ep(int en);
void update_pid_ep(int en);
void set_control_reg_pid0_ep(int en);
void set_control_reg_pid1_ep(int en);
void set_control_reg_stall_ep(int en);
void set_stall_flag();
int  get_stall_flag();
void reset_stall_flag();

void set7SEG(char n);

void wait_configured();
// static data
extern st_usbs_env g_usbs_env;

////////////////////////////////////////
extern unsigned char g_usb_device_desc[];
extern unsigned char g_usb_conf_desc[];
extern unsigned char g_usb_string_desc0[];
extern unsigned char g_usb_string_desc1[];
extern unsigned char g_usb_string_desc2[];
extern unsigned char g_usb_string_desc3[];

#endif /* USBS_BASE_H_ */
