//=======================================================================
//                         USB Target CDC sample
//
// File:
//   usbs_hw_reg.h
//
// Abstract:
//   Register mapping
//
//  Created:
//    27 April 2015
//======================================================================
//
// Copyright (c) 2015, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id$

#ifndef USBS_HW_REG_H
#define USBS_HW_REG_H

#include <system.h>

// USB port select macro
#define USB_ENDPOINT0_CONTROL_REG               (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x40))
#define USB_ENDPOINT0_STATUS_REG                (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x41))
#define USB_ENDPOINT0_TRANSTYPE_STATUS_REG      (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x42))
#define USB_ENDPOINT0_NAK_TRANSTYPE_STATUS_REG  (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x43))
#define USB_ENDPOINT1_CONTROL_REG               (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x44))
#define USB_ENDPOINT1_STATUS_REG                (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x45))
#define USB_ENDPOINT1_TRANSTYPE_STATUS_REG      (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x46))
#define USB_ENDPOINT1_NAK_TRANSTYPE_STATUS_REG  (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x47))
#define USB_ENDPOINT2_CONTROL_REG               (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x48))
#define USB_ENDPOINT2_STATUS_REG                (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x49))
#define USB_ENDPOINT2_TRANSTYPE_STATUS_REG      (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x4a))
#define USB_ENDPOINT2_NAK_TRANSTYPE_STATUS_REG  (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x4b))
#define USB_ENDPOINT3_CONTROL_REG               (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x4c))
#define USB_ENDPOINT3_STATUS_REG                (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x4d))
#define USB_ENDPOINT3_TRANSTYPE_STATUS_REG      (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x4e))
#define USB_ENDPOINT3_NAK_TRANSTYPE_STATUS_REG  (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x4f))
#define USB_SC_CONTROL_REG                      (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x50))
#define USB_SC_LINE_STATUS_REG                  (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x51))
#define USB_SC_INTERRUPT_STATUS_REG             (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x52))
#define USB_SC_INTERRUPT_MASK_REG               (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x53))
#define USB_SC_ADDRESS                          (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x54))
#define USB_SC_FRAME_NUM_MSP                    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x55))
#define USB_SC_FRAME_NUM_LSP                    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x56))

#define USB_EP0_RX_FIFO_DATA                    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x60))
#define USB_EP0_RX_FIFO_DATA_COUNT_MSB          (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x62))
#define USB_EP0_RX_FIFO_DATA_COUNT_LSB          (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x63))
#define USB_EP0_RX_FIFO_CONTROL_REG             (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x64))
#define USB_EP0_TX_FIFO_DATA                    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x70))
#define USB_EP0_TX_FIFO_CONTROL_REG             (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x74))
#define USB_EP1_RX_FIFO_DATA                    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x80))
#define USB_EP1_RX_FIFO_DATA_COUNT_MSB          (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x82))
#define USB_EP1_RX_FIFO_DATA_COUNT_LSB          (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x83))
#define USB_EP1_RX_FIFO_CONTROL_REG             (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x84))
#define USB_EP1_TX_FIFO_DATA                    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x90))
#define USB_EP1_TX_FIFO_CONTROL_REG             (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0x94))
#define USB_EP2_RX_FIFO_DATA                    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xa0))
#define USB_EP2_RX_FIFO_DATA_COUNT_MSB          (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xa2))
#define USB_EP2_RX_FIFO_DATA_COUNT_LSB          (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xa3))
#define USB_EP2_RX_FIFO_CONTROL_REG             (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xa4))
#define USB_EP2_TX_FIFO_DATA                    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xb0))
#define USB_EP2_TX_FIFO_CONTROL_REG             (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xb4))
#define USB_EP3_RX_FIFO_DATA                    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xc0))
#define USB_EP3_RX_FIFO_DATA_COUNT_MSB          (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xc2))
#define USB_EP3_RX_FIFO_DATA_COUNT_LSB          (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xc3))
#define USB_EP3_RX_FIFO_CONTROL_REG             (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xc4))
#define USB_EP3_TX_FIFO_DATA                    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xd0))
#define USB_EP3_TX_FIFO_CONTROL_REG             (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xd4))


#define USB_HOST_SLAVE_CONTROL_REG    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xe0))
#define USB_HOST_SLAVE_VERSION_REG    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xe1))

#define USB_CDC_INT_MASK_REG                    (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xf0))
#define USB_CDC_STATUS_REG                      (*(volatile unsigned char *)(FM_USBS_WRAPPER_0_BASE+0xf1))


#define USBS_ENDPOINT_ENABLE_BIT 0x1
#define USBS_ENDPOINT_READY_BIT 0x2

#define USBS_SC_TRANS_DONE_BIT 0x1
#define USBS_SC_RESUME_INT_BIT 0x2
#define USBS_SC_RESET_EVENT_BIT 0x4
#define USBS_SC_SOF_RECEIVED_BIT 0x8
#define USBS_SC_NAK_SENT_BIT 0x10
#define USBS_VBUS_DET_INT_BIT 0x20

#endif

