//=======================================================================
//                         DE0 USB CDC Target
//
// File:
//   EP0Cdc.v
//
// Abstract:
//   Endpoint0 Controller
//
//  Created:
//    27 April 2015
//======================================================================
//
// Copyright (c) 2015, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id: 34dc9821c24831b35445520591796a1e9da60c27 $

`include "timescale.v"
`include "usbHostSlaveReg_define.v"
`include "usbCdcDevice_define.v"

module EP0Cdc (clk, initComplete, memAddr, memData, memRdEn, rst, wb_ack, wb_addr, wb_data_i, wb_data_o, wb_stb, wb_we, wbBusGnt, wbBusReq,stateDefault,stateAddress,stateConfigured);
input   clk;
input   [7:0]memData;
input   rst;
input   wb_ack;
input   [7:0]wb_data_i;
input   wbBusGnt;
output  initComplete;
output  [7:0]memAddr;
output  memRdEn;
output  [7:0]wb_addr;
output  [7:0]wb_data_o;
output  wb_stb;
output  wb_we;
output  wbBusReq;
output stateDefault;
output stateAddress;
output stateConfigured;
   
wire    clk;
reg     initComplete, next_initComplete;
reg     [7:0]memAddr, next_memAddr;
wire    [7:0]memData;
reg     memRdEn, next_memRdEn;
wire    rst;
wire    wb_ack;
reg     [7:0]wb_addr, next_wb_addr;
wire    [7:0]wb_data_i;
reg     [7:0]wb_data_o, next_wb_data_o;
reg     wb_stb, next_wb_stb;
reg     wb_we, next_wb_we;
wire    wbBusGnt;
reg     wbBusReq, next_wbBusReq;

// diagram signals declarations
reg bm_req_dir, next_bm_req_dir;
reg  [4:0]bm_req_recp, next_bm_req_recp;
reg  [1:0]bm_req_type, next_bm_req_type;
reg  [7:0]bRequest, next_bRequest;
reg  [7:0]cnt, next_cnt;
reg dataSeq, next_dataSeq;
reg  [7:0]epStatus, next_epStatus;
reg  [7:0]epTransType, next_epTransType;
reg localRst, next_localRst;
reg  [15:0]rxDataSize, next_rxDataSize;
reg transDone, next_transDone;
reg  [7:0]txDataIndex, next_txDataIndex;
reg  [7:0]txDataSize, next_txDataSize;
reg  [7:0]txPacketRemSize, next_txPacketRemSize;
reg updateUSBAddress, next_updateUSBAddress;
reg  [7:0]USBAddress, next_USBAddress;
reg  [15:0]wIndex, next_wIndex;
reg  [15:0]wLength, next_wLength;
reg  [15:0]wValue, next_wValue;

reg  stateConfigured, next_stateConfigured;
reg  stateAddress, next_stateAddress;
reg  stateDefault, next_stateDefault;
reg  updateSetLineCoding, next_updateSetLineCoding;
reg  updateGetLineCoding, next_updateGetLineCoding;

wire [7:0] w_memData_mux;
   
   
// BINARY ENCODED state machine: EP0St
// State codes definitions:
localparam P_START        = 7'h00;
localparam P_INIT_WT_VBUS_GNT  = 7'h01;
localparam P_INIT_WT_VBUS = 7'h02;
localparam P_INIT_DEL     = 7'h03;
localparam P_INIT_CONN    = 7'h04;
localparam P_INIT_WT_CONN_GNT = 7'h05;
localparam P_INIT_WT_CONN     = 7'h06;
localparam P_INIT_CONN_DEL    = 7'h07;
localparam P_INIT_FIN         = 7'h08;
localparam P_DO_TRANS_WT_GNT  = 7'h09;
localparam P_DO_TRANS_TX_EMPTY = 7'h0a;
localparam P_DO_TRANS_WR_TX_FIFO = 7'h0b;
localparam P_DO_TRANS_RD_MEM     = 7'h0c;
localparam P_DO_TRANS_CHK_TX_DONE = 7'h0d;
localparam P_DO_TRANS_TRANS_GO    = 7'h0e;
localparam P_DO_TRANS_WT_TRANS_DONE_WT_GNT = 7'h0f;
localparam P_DO_TRANS_WT_TRANS_DONE_GET_RDY_STS = 7'h10;
localparam P_DO_TRANS_WT_TRANS_DONE_WT_UNGNT = 7'h11;
localparam P_DO_TRANS_WT_TRANS_DONE_CHK_DONE = 7'h12;
localparam P_CHK_TRANS_RD_STAT = 7'h13;
localparam P_CHK_TRANS_WT_GNT = 7'h14;
localparam P_CHK_TRANS_RD_RX_SIZE1 = 7'h15;
localparam P_CHK_TRANS_RD_RX_SIZE2 = 7'h16;
localparam P_CHK_TRANS_RD_TRANS_TYPE = 7'h17;
localparam P_CHK_TRANS_WT_UNGNT = 7'h18;
localparam P_SETUP_CHK_ERR = 7'h19;
localparam P_SETUP_GET_DATA_DAT1 = 7'h1a;
localparam P_SETUP_GET_DATA_WT_GNT = 7'h1b;
localparam P_SETUP_GET_DATA_DAT2 = 7'h1c;
localparam P_SETUP_GET_DATA_DAT3 = 7'h1d;
localparam P_SETUP_GET_DATA_DAT4 = 7'h1e;
localparam P_SETUP_GET_DATA_DAT6 = 7'h1f;
localparam P_SETUP_GET_DATA_DAT5 = 7'h20;
localparam P_SETUP_GET_DATA_DAT8 = 7'h21;
localparam P_SETUP_GET_DATA_DAT7 = 7'h22;
localparam P_SETUP_GET_DATA_WT_UNGNT = 7'h23;
localparam P_SETUP_GET_STAT = 7'h24;
localparam P_SETUP_SET_ADDR = 7'h25;
localparam P_SETUP_GET_DESC_S1 = 7'h26;
localparam P_SETUP_CHK_MAX_LEN = 7'h27;
localparam P_OUT_CHK_SEQ = 7'h28;
localparam P_IN_CHK_ACK = 7'h29;
localparam P_IN_SET_PTR = 7'h2a;
localparam P_IN_SET_ADDR = 7'h2b;
localparam P_IN_WT_GNT = 7'h2c;
localparam P_IN_WT_UNGNT = 7'h2d;
localparam P_DO_TRANS_RX_EMPTY = 7'h2e;
localparam P_DO_TRANS_WT_TRANS_DONE_DEL = 7'h2f;
localparam P_DO_TRANS_DEL = 7'h30;
localparam P_SETUP_PTR_SET = 7'h31;
localparam P_SETUP_SET_CONFIG = 7'h32;
localparam P_SETUP_SET_CONTROL_LINE_STATE = 7'h33;
localparam P_SETUP_SET_LINE_CODING = 7'h34;
localparam P_OUT_GET_SET_LINE_CODING_DAT = 7'h35;
localparam P_SETUP_GET_LINE_CODING     = 7'h36;
localparam P_OUT_TRANS_WT_GNT          = 7'h37;
localparam P_OUT_SET_LINE_CODING_DAT   = 7'h38;
localparam P_OUT_TRANS_WT_UNGNT        = 7'h39;
localparam P_OUT_GET_SET_LINE_CODING_ACK = 7'h3a;
localparam P_INIT_WT_RST_GNT  = 7'h3b;
localparam P_INIT_WT_RST      = 7'h3c;
localparam P_INIT_RST         = 7'h3d;

reg [6:0]CurrState_EP0St, NextState_EP0St;

reg [7:0] SetLineCodingBuf[6:0];
reg [7:0] SetLineCodingBuf_1z;
  
assign w_memData_mux = SetLineCodingBuf[memAddr[2:0]];
   
// Machine: EP0St

// NextState logic (combinatorial)
always @* begin
  NextState_EP0St = CurrState_EP0St;
  // Set default values for outputs and signals
  next_wb_addr = wb_addr;
  next_wb_data_o = wb_data_o;
  next_wb_stb = wb_stb;
  next_wb_we = wb_we;
  next_cnt = cnt;
  next_wbBusReq = wbBusReq;
  next_initComplete = initComplete;
  next_memAddr = memAddr;
  next_memRdEn = memRdEn;
  next_txDataSize = txDataSize;
  next_txDataIndex = txDataIndex;
  next_transDone = transDone;
  next_epStatus = epStatus;
  next_rxDataSize = rxDataSize;
  next_epTransType = epTransType;
  next_bm_req_dir = bm_req_dir;
  next_bm_req_type = bm_req_type;
  next_bm_req_recp = bm_req_recp;
  next_bRequest = bRequest;
  next_wValue = wValue;
  next_wIndex = wIndex;
  next_wLength = wLength;
  next_txPacketRemSize = txPacketRemSize;
  next_USBAddress = USBAddress;
  next_updateUSBAddress = updateUSBAddress;
  next_dataSeq = dataSeq;
  next_stateDefault = stateDefault;
  next_stateAddress = stateAddress;
  next_stateConfigured = stateConfigured;
  next_updateSetLineCoding = updateSetLineCoding;
  next_updateGetLineCoding = updateGetLineCoding;
  case (CurrState_EP0St)  // synopsys parallel_case full_case
    P_START:
    begin
      next_initComplete = 1'b0;
      next_wbBusReq = 1'b0;
      next_wb_addr = 8'h00;
      next_wb_data_o = 8'h00;
      next_wb_stb = 1'b0;
      next_wb_we = 1'b0;
      next_txPacketRemSize = 8'h00;
      next_txDataSize = 8'h00;
      next_txDataIndex = 8'h00;
      next_epTransType = 8'h00;
      next_epStatus = 8'h00;
      next_rxDataSize = 16'h0000;
      next_cnt = 8'h00;
      next_memRdEn = 1'b0;
      next_memAddr = 8'h00;
      next_updateUSBAddress = 1'b0;
      next_updateSetLineCoding = 1'b0;
      next_updateGetLineCoding = 1'b0;
      next_stateDefault = 1'b1;
      next_stateAddress = 1'b0;
      next_stateConfigured = 1'b0;
      next_transDone = 1'b0;
      next_bm_req_type = 2'b00;
      next_bm_req_dir = 1'b0;
      next_bm_req_recp = 5'b00000;
      next_bRequest = 8'h00;
      next_wLength = 16'h0000;
      next_wIndex = 16'h0000;
      next_wValue = 16'h0000;
      next_dataSeq = 1'b0;
      next_USBAddress = 8'h00;
      NextState_EP0St = P_INIT_WT_RST_GNT;
    end
    P_INIT_WT_RST_GNT:
    begin
      next_wbBusReq = 1'b1;
      if (wbBusGnt == 1'b1)
      begin
        NextState_EP0St = P_INIT_RST;
      end
    end
    P_INIT_RST:
    begin
      next_wb_addr = `RA_HOST_SLAVE_MODE;
      next_wb_data_o = 8'h2;
      //reset usbHostSlave
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        next_wbBusReq = 1'b0;
        NextState_EP0St = P_INIT_WT_RST;
        next_wb_stb = 1'b0;
        next_cnt = 8'h00;
      end
    end
    P_INIT_WT_RST:
    begin
      next_cnt = cnt + 1'b1;
      if (cnt == 8'hff)
      begin
        NextState_EP0St = P_INIT_WT_VBUS_GNT;
      end
    end
    P_CHK_TRANS_RD_STAT:
    begin
      next_wb_addr = `RA_EP0_STATUS_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_CHK_TRANS_RD_RX_SIZE1;
        next_wb_stb = 1'b0;
        next_epStatus = wb_data_i;
      end
    end
    P_CHK_TRANS_WT_GNT:
    begin
      if (wbBusGnt == 1'b1)
      begin
        NextState_EP0St = P_CHK_TRANS_RD_STAT;
      end
    end
    P_CHK_TRANS_RD_RX_SIZE1:
    begin
      next_wb_addr = `RA_EP0_RX_FIFO_DATA_COUNT_MSB;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_CHK_TRANS_RD_RX_SIZE2;
        next_wb_stb = 1'b0;
        next_rxDataSize[15:8] = wb_data_i;
      end
    end
    P_CHK_TRANS_RD_RX_SIZE2:
    begin
      next_wb_addr = `RA_EP0_RX_FIFO_DATA_COUNT_LSB;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_CHK_TRANS_RD_TRANS_TYPE;
        next_wb_stb = 1'b0;
        next_rxDataSize[7:0] = wb_data_i;
      end
    end
    P_CHK_TRANS_RD_TRANS_TYPE:
    begin
      next_wb_addr = `RA_EP0_TRANSTYPE_STATUS_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_CHK_TRANS_WT_UNGNT;
        next_wb_stb = 1'b0;
        next_epTransType = wb_data_i;
      end
    end
    P_CHK_TRANS_WT_UNGNT:
    begin
      next_wbBusReq = 1'b0;
      if ((wbBusGnt == 1'b0) && ((epStatus & 8'h0f) != 8'h00))
      begin
        NextState_EP0St = P_DO_TRANS_WT_GNT;
      end
      else if ((wbBusGnt == 1'b0) && (epTransType == `SC_SETUP_TRANS))
      begin
        NextState_EP0St = P_SETUP_CHK_ERR;
      end
      else if ((wbBusGnt == 1'b0) && (epTransType == `SC_IN_TRANS))
      begin
        NextState_EP0St = P_IN_CHK_ACK;
      end
      else if ((wbBusGnt == 1'b0) && (epTransType == `SC_OUTDATA_TRANS))
      begin
        NextState_EP0St = P_OUT_CHK_SEQ;
      end
      else if (wbBusGnt == 1'b0)
      begin
        NextState_EP0St = P_DO_TRANS_WT_GNT;
      end
    end
    P_DO_TRANS_WT_GNT:
    begin
      next_wbBusReq = 1'b1;
      if (wbBusGnt == 1'b1)
      begin
        NextState_EP0St = P_DO_TRANS_TX_EMPTY;
      end
    end
    P_DO_TRANS_TX_EMPTY:
    begin
      next_wb_addr = `RA_EP0_TX_FIFO_CONTROL_REG;
      next_wb_data_o = 8'h01;
      //force tx fifo empty
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_DO_TRANS_RX_EMPTY;
        next_wb_stb = 1'b0;
      end
    end
    P_DO_TRANS_WR_TX_FIFO:
    begin
      next_wb_data_o =   (updateGetLineCoding) ? SetLineCodingBuf_1z : memData;
      next_wb_addr = `RA_EP0_TX_FIFO_DATA_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_DO_TRANS_CHK_TX_DONE;
        next_wb_stb = 1'b0;
      end
    end
    P_DO_TRANS_RD_MEM:
    begin
      next_memAddr = txDataIndex;
      next_memRdEn = 1'b1;
      next_txDataSize = txDataSize - 1'b1;
      next_txDataIndex = txDataIndex + 1'b1;
      NextState_EP0St = P_DO_TRANS_DEL;
    end
    P_DO_TRANS_CHK_TX_DONE:
    begin
      if (txDataSize == 8'h00)
      begin
        NextState_EP0St = P_DO_TRANS_TRANS_GO;
      end
      else
      begin
        NextState_EP0St = P_DO_TRANS_RD_MEM;
      end
    end
    P_DO_TRANS_TRANS_GO:
    begin
      next_wb_addr = `RA_EP0_CONTROL_REG;
      if (dataSeq == 1'b1)
      next_wb_data_o = 8'h07;
      else
      next_wb_data_o = 8'h03;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        next_dataSeq = ~dataSeq;
        NextState_EP0St = P_DO_TRANS_WT_TRANS_DONE_WT_GNT;
        next_wb_stb = 1'b0;
        next_transDone = 1'b0;
      end
    end
    P_DO_TRANS_RX_EMPTY:
    begin
      next_wb_addr = `RA_EP0_RX_FIFO_CONTROL_REG;
      next_wb_data_o = 8'h01;
      //force rx fifo empty
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if ((wb_ack == 1'b1) && (txDataSize != 8'h00))
      begin
        NextState_EP0St = P_DO_TRANS_RD_MEM;
        next_wb_stb = 1'b0;
      end
      else if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_DO_TRANS_TRANS_GO;
        next_wb_stb = 1'b0;
      end
    end
    P_DO_TRANS_DEL:
    begin
      next_memRdEn = 1'b0;
      NextState_EP0St = P_DO_TRANS_WR_TX_FIFO;
    end
    P_DO_TRANS_WT_TRANS_DONE_WT_GNT:
    begin
      next_wbBusReq = 1'b1;
      if (wbBusGnt == 1'b1)
      begin
        NextState_EP0St = P_DO_TRANS_WT_TRANS_DONE_GET_RDY_STS;
      end
    end
    P_DO_TRANS_WT_TRANS_DONE_GET_RDY_STS:
    begin
      next_wb_addr = `RA_EP0_CONTROL_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_DO_TRANS_WT_TRANS_DONE_WT_UNGNT;
        next_wb_stb = 1'b0;
        next_transDone = ~wb_data_i[`ENDPOINT_READY_BIT];
      end
    end
    P_DO_TRANS_WT_TRANS_DONE_WT_UNGNT:
    begin
      next_wbBusReq = 1'b0;
      if (wbBusGnt == 1'b0)
      begin
        NextState_EP0St = P_DO_TRANS_WT_TRANS_DONE_CHK_DONE;
      end
    end
    P_DO_TRANS_WT_TRANS_DONE_CHK_DONE:
    begin
      if (transDone == 1'b1)
      begin
        NextState_EP0St = P_CHK_TRANS_WT_GNT;
        next_wbBusReq = 1'b1;
      end
      else
      begin
        NextState_EP0St = P_DO_TRANS_WT_TRANS_DONE_DEL;
        next_cnt = 8'h00;
      end
    end
    P_DO_TRANS_WT_TRANS_DONE_DEL:
    begin
      next_cnt = cnt + 1'b1;
      if (cnt == `ONE_USEC_DEL)
      begin
        NextState_EP0St = P_DO_TRANS_WT_TRANS_DONE_WT_GNT;
      end
    end
    P_SETUP_CHK_ERR:
    begin
      if (rxDataSize != 16'h0008)
      begin
        NextState_EP0St = P_DO_TRANS_WT_GNT;
      end
      else
      begin
        NextState_EP0St = P_SETUP_GET_DATA_WT_GNT;
        next_wbBusReq = 1'b1;
        next_txDataSize = 8'h00;
        next_txPacketRemSize = 8'h00;
        //default tx packet size
        next_dataSeq = 1'b1;
        next_wb_addr = `RA_EP0_RX_FIFO_DATA_REG;
        next_wb_we = 1'b0;
      end
    end
    P_SETUP_GET_STAT:
    begin
      if (bm_req_type == 2'b00)  begin
      next_txPacketRemSize = 8'h02;
      if (bm_req_recp == 5'b00000)
      next_txDataIndex = `ONE_ZERO_STAT_INDEX;
      else
      next_txDataIndex = `ZERO_ZERO_STAT_INDEX;
      end
      else if (bm_req_type == 2'b10) begin
      next_txDataIndex = `VENDOR_DATA_STAT_INDEX;
      next_txPacketRemSize = 8'h02;
      end
      NextState_EP0St = P_SETUP_CHK_MAX_LEN;
    end
    P_SETUP_GET_LINE_CODING:
    begin
      next_txPacketRemSize = 8'h07;
      next_txDataIndex = 8'h00;
      next_updateGetLineCoding = 1'b1;
      NextState_EP0St = P_SETUP_CHK_MAX_LEN;
    end
    P_SETUP_SET_ADDR:
    begin
      if ( (wValue[15:7] == {9{1'b0}}) && (wIndex == 16'h0000) && (wLength == 16'h0000) ) begin
      next_USBAddress = wValue[7:0];
      next_updateUSBAddress = 1'b1;
      next_stateDefault = 1'b0;
      next_stateAddress = 1'b1;
      end
      NextState_EP0St = P_SETUP_CHK_MAX_LEN;
    end
    P_SETUP_SET_CONFIG:
    begin
      if ( (wValue[15:8] == {8{1'b0}}) && (wIndex == 16'h0000) && (wLength == 16'h0000) ) begin
      next_stateAddress = 1'b0;
      next_stateConfigured = 1'b1;
      end
      NextState_EP0St = P_SETUP_CHK_MAX_LEN;
    end
    P_SETUP_SET_CONTROL_LINE_STATE:
    begin
      NextState_EP0St = P_SETUP_CHK_MAX_LEN;
    end
    P_SETUP_SET_LINE_CODING:
    begin
      next_updateSetLineCoding = 1'b1;
      NextState_EP0St = P_SETUP_CHK_MAX_LEN;
    end
    P_SETUP_CHK_MAX_LEN:
    begin
      if (txPacketRemSize > wLength)
      next_txPacketRemSize = wLength;
      NextState_EP0St = P_SETUP_PTR_SET;
    end
    P_SETUP_PTR_SET:
    begin
      if (txPacketRemSize > `MAX_RESP_SIZE) begin
        next_txDataSize = `MAX_RESP_SIZE;
        next_txPacketRemSize = txPacketRemSize - `MAX_RESP_SIZE;
      end
      else begin
        next_txDataSize = txPacketRemSize;
        next_txPacketRemSize = 8'h00;
      end
      NextState_EP0St = P_DO_TRANS_WT_GNT;
    end
    P_SETUP_GET_DATA_DAT1:
    begin
      next_wb_stb = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_SETUP_GET_DATA_DAT2;
        next_wb_stb = 1'b0;
        next_bm_req_dir = wb_data_i[7];
        next_bm_req_type = wb_data_i[6:5];
        next_bm_req_recp = wb_data_i[4:0];
      end
    end
    P_SETUP_GET_DATA_WT_GNT:
    begin
      if (wbBusGnt == 1'b1)
      begin
        NextState_EP0St = P_SETUP_GET_DATA_DAT1;
      end
    end
    P_SETUP_GET_DATA_DAT2:
    begin
      next_wb_stb = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_SETUP_GET_DATA_DAT3;
        next_wb_stb = 1'b0;
        next_bRequest = wb_data_i;
      end
    end
    P_SETUP_GET_DATA_DAT3:
    begin
      next_wb_stb = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_SETUP_GET_DATA_DAT4;
        next_wb_stb = 1'b0;
        next_wValue[7:0] = wb_data_i;
      end
    end
    P_SETUP_GET_DATA_DAT4:
    begin
      next_wb_stb = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_SETUP_GET_DATA_DAT5;
        next_wb_stb = 1'b0;
        next_wValue[15:8] = wb_data_i;
      end
    end
    P_SETUP_GET_DATA_DAT6:
    begin
      next_wb_stb = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_SETUP_GET_DATA_DAT7;
        next_wb_stb = 1'b0;
        next_wIndex[15:8] = wb_data_i;
      end
    end
    P_SETUP_GET_DATA_DAT5:
    begin
      next_wb_stb = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_SETUP_GET_DATA_DAT6;
        next_wb_stb = 1'b0;
        next_wIndex[7:0] = wb_data_i;
      end
    end
    P_SETUP_GET_DATA_DAT8:
    begin
      next_wb_stb = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_SETUP_GET_DATA_WT_UNGNT;
        next_wb_stb = 1'b0;
        next_wLength[15:8] = wb_data_i;
        next_wbBusReq = 1'b0;
      end
    end
    P_SETUP_GET_DATA_DAT7:
    begin
      next_wb_stb = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_SETUP_GET_DATA_DAT8;
        next_wb_stb = 1'b0;
        next_wLength[7:0] = wb_data_i;
      end
    end
    P_SETUP_GET_DATA_WT_UNGNT:
    begin
      // clear flags
      next_updateUSBAddress = 1'b0;
      next_updateSetLineCoding = 1'b0;
      next_updateGetLineCoding = 1'b0;

      if ((wbBusGnt == 1'b0) && (bRequest == `GET_STATUS))
      begin
        NextState_EP0St = P_SETUP_GET_STAT;
      end
      else if ((wbBusGnt == 1'b0) && (bRequest == `GET_DESCRIPTOR))
      begin
        NextState_EP0St = P_SETUP_GET_DESC_S1;
      end
      else if ((wbBusGnt == 1'b0) && (bRequest == `SET_ADDRESS))
      begin
        NextState_EP0St = P_SETUP_SET_ADDR;
      end
      else if ((wbBusGnt == 1'b0) && (bRequest == `SET_CONFIG))
      begin
        NextState_EP0St = P_SETUP_SET_CONFIG;
      end
      // CDC Class Request
      else if ((wbBusGnt == 1'b0) && (bRequest == `SET_LINE_CODING))
      begin
        NextState_EP0St = P_SETUP_SET_LINE_CODING;
      end
      else if ((wbBusGnt == 1'b0) && (bRequest == `GET_LINE_CODING))
      begin
        NextState_EP0St = P_SETUP_GET_LINE_CODING;
      end
      else if ((wbBusGnt == 1'b0) && (bRequest == `SET_CONTROL_LINE_STATE))
      begin
        NextState_EP0St = P_SETUP_SET_CONTROL_LINE_STATE;
      end
      else if (wbBusGnt == 1'b0)
      begin
        NextState_EP0St = P_DO_TRANS_WT_GNT;
      end
    end
    P_SETUP_GET_DESC_S1:
    begin
      case (wValue[15:8])
      `DEV_DESC: begin
      next_txPacketRemSize = `DEV_DESC_SIZE;
      next_txDataIndex = `DEV_DESC_INDEX;
      end
      `CFG_DESC: begin
      next_txPacketRemSize = `CFG_DESC_SIZE;
      next_txDataIndex = `CFG_DESC_INDEX;
      end
 //     `REP_DESC: begin
 //     next_txPacketRemSize = `REP_DESC_SIZE;
 //     next_txDataIndex = `REP_DESC_INDEX;
 //     end
      `STRING_DESC: begin
      case (wValue[3:0])
      4'h0: begin
      next_txPacketRemSize = `LANGID_DESC_SIZE;
      next_txDataIndex = `LANGID_DESC_INDEX;
      end
      4'h1: begin
      next_txPacketRemSize = `STRING1_DESC_SIZE;
      next_txDataIndex = `STRING1_DESC_INDEX;
      end
      4'h2: begin
      next_txPacketRemSize = `STRING2_DESC_SIZE;
      next_txDataIndex = `STRING2_DESC_INDEX;
      end
      4'h3: begin
      next_txPacketRemSize = `STRING3_DESC_SIZE;
      next_txDataIndex = `STRING3_DESC_INDEX;
      end
      endcase
      end
      endcase
      NextState_EP0St = P_SETUP_CHK_MAX_LEN;
    end
    P_IN_CHK_ACK:
    begin
      if (epStatus[`SC_ACK_RXED_BIT] != 1'b1)
      begin
        NextState_EP0St = P_DO_TRANS_WT_GNT;
      end
      else if (updateUSBAddress == 1'b1)
      begin
        NextState_EP0St = P_IN_WT_GNT;
      end
      else if (updateSetLineCoding == 1'b1)
      begin
        NextState_EP0St = P_DO_TRANS_WT_GNT;
      end
      else
      begin
        NextState_EP0St = P_IN_SET_PTR;
      end
    end
    P_IN_SET_PTR:
    begin
      if (txPacketRemSize > `MAX_RESP_SIZE) begin
      next_txDataSize = `MAX_RESP_SIZE;
      next_txPacketRemSize = txPacketRemSize - `MAX_RESP_SIZE;
      end
      else begin
      next_txDataSize = txPacketRemSize;
      next_txPacketRemSize = 8'h00;
      end
      NextState_EP0St = P_DO_TRANS_WT_GNT;
    end
    P_IN_SET_ADDR:
    begin
      next_wb_addr = `RA_SC_ADDRESS;
      next_wb_data_o = USBAddress;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_IN_WT_UNGNT;
        next_wb_stb = 1'b0;
        next_wbBusReq = 1'b0;
      end
    end
    P_IN_WT_GNT:
    begin
      next_wbBusReq = 1'b1;
      next_updateUSBAddress = 1'b0;
      if (wbBusGnt == 1'b1)
      begin
        NextState_EP0St = P_IN_SET_ADDR;
      end
    end
    P_IN_WT_UNGNT:
    begin
      if (wbBusGnt == 1'b0)
      begin
        NextState_EP0St = P_IN_SET_PTR;
      end
    end
    P_OUT_CHK_SEQ:
    begin
      if (epStatus[`SC_DATA_SEQUENCE_BIT] != dataSeq)
      begin
        NextState_EP0St = P_DO_TRANS_WT_GNT;
      end
      else
      begin
        next_dataSeq = ~dataSeq;
        if (updateSetLineCoding) begin
          NextState_EP0St = P_OUT_TRANS_WT_GNT;
        end else begin	   
          NextState_EP0St = P_DO_TRANS_WT_GNT;
        end
      end
    end
    P_OUT_TRANS_WT_GNT:
    begin
      next_wbBusReq = 1'b1;
      if (wbBusGnt == 1'b1)
      begin
        next_wb_stb = 1'b1;
        next_wb_addr = `RA_EP0_RX_FIFO_DATA_REG;
        next_wb_we = 1'b0;
        NextState_EP0St = P_OUT_GET_SET_LINE_CODING_DAT;
      end
    end
    P_OUT_GET_SET_LINE_CODING_DAT:
    begin
      if (wb_ack) begin	 
        next_rxDataSize = rxDataSize - 1'b1;
        if (rxDataSize == 16'h0) begin
          next_wb_stb = 1'b0;
          NextState_EP0St = P_OUT_TRANS_WT_UNGNT;
        end else begin
          next_wb_stb = 1'b0;
          NextState_EP0St = P_OUT_GET_SET_LINE_CODING_ACK;
	end
      end
    end
    P_OUT_GET_SET_LINE_CODING_ACK:
    begin
      if (!wb_ack) begin
        next_wb_stb = 1'b1;
        NextState_EP0St = P_OUT_GET_SET_LINE_CODING_DAT;
      end
    end
    P_OUT_TRANS_WT_UNGNT:
     begin
      next_wbBusReq = 1'b0;
      if (wbBusGnt == 1'b0)
      begin
        NextState_EP0St =  P_DO_TRANS_WT_GNT;
      end
    end
    P_INIT_WT_VBUS_GNT:
    begin
      next_wbBusReq = 1'b1;
      if (wbBusGnt == 1'b1)
      begin
        NextState_EP0St = P_INIT_WT_VBUS;
      end
    end
    P_INIT_WT_VBUS:
    begin
      next_wb_addr = `RA_SC_LINE_STATUS_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1) begin
        if (wb_data_i[`VBUS_PRES_BIT] == 1'b1) begin
          NextState_EP0St = P_INIT_CONN;
          next_wb_stb = 1'b0;
        end else begin
          next_wb_stb = 1'b0;
          next_wbBusReq = 1'b0;
          next_cnt = 8'h0;
          NextState_EP0St = P_INIT_DEL;
        end
      end
    end
    P_INIT_DEL:
    begin
      next_cnt = cnt + 1'b1;
      if (cnt == `ONE_USEC_DEL)
      begin
        NextState_EP0St = P_INIT_WT_VBUS_GNT;
      end
    end
    P_INIT_FIN:
    begin
      next_wbBusReq = 1'b0;
      next_initComplete = 1'b1;
      if (wbBusGnt == 1'b0)
      begin
        NextState_EP0St = P_DO_TRANS_WT_GNT;
      end
    end
    P_INIT_CONN:
    begin
      next_wb_addr = `RA_SC_CONTROL_REG;
      next_wb_data_o = 8'h71;
      //connect to host, full speed
      next_wb_stb = 1'b1;
      next_wb_we = 1'b1;
      if (wb_ack == 1'b1)
      begin
        NextState_EP0St = P_INIT_WT_CONN;
        next_wb_stb = 1'b0;
      end
    end
    P_INIT_WT_CONN_GNT:
    begin
      next_wbBusReq = 1'b1;
      if (wbBusGnt == 1'b1)
      begin
        NextState_EP0St = P_INIT_WT_CONN;
      end
    end
    P_INIT_WT_CONN:
    begin
      next_wb_addr = `RA_SC_LINE_STATUS_REG;
      next_wb_stb = 1'b1;
      next_wb_we = 1'b0;
      if (wb_ack == 1'b1) begin
        if (wb_data_i[1:0] == `FULL_SPEED_CONNECT) begin
          NextState_EP0St = P_INIT_FIN;
          next_wb_stb = 1'b0;
        end else begin
          next_wbBusReq = 1'b0;
          next_cnt = 8'h0;
          NextState_EP0St = P_INIT_CONN_DEL;
          next_wb_stb = 1'b0;
	end
      end
    end
    P_INIT_CONN_DEL:
    begin
      next_cnt = cnt + 1'b1;
      if (cnt == `ONE_USEC_DEL)
      begin
        NextState_EP0St = P_INIT_WT_CONN_GNT;
      end
    end
  endcase
end

// Current State Logic (sequential)
always @ (posedge clk)
begin
  if (rst == 1'b1)
    CurrState_EP0St <= P_START;
  else
    CurrState_EP0St <= NextState_EP0St;
end

// Registered outputs logic
always @ (posedge clk)
begin
  if (rst == 1'b1)
  begin
    wb_addr <= 8'h00;
    wb_data_o <= 8'h00;
    wb_stb <= 1'b0;
    wb_we <= 1'b0;
    wbBusReq <= 1'b0;
    initComplete <= 1'b0;
    memAddr <= 8'h00;
    memRdEn <= 1'b0;
    cnt <= 8'h00;
    txDataSize <= 8'h00;
    txDataIndex <= 8'h00;
    transDone <= 1'b0;
    epStatus <= 8'h00;
    rxDataSize <= 16'h0000;
    epTransType <= 8'h00;
    bm_req_dir <= 1'b0;
    bm_req_type <= 2'b00;
    bm_req_recp <= 5'b00000;
    bRequest <= 8'h00;
    wValue <= 16'h0000;
    wIndex <= 16'h0000;
    wLength <= 16'h0000;
    txPacketRemSize <= 8'h00;
    USBAddress <= 8'h00;
    updateUSBAddress <= 1'b0;
    updateSetLineCoding <= 1'b0;
    updateGetLineCoding <= 1'b0;
    dataSeq <= 1'b0;
    stateDefault <= 1'b0;
    stateAddress <= 1'b0;
    stateConfigured <= 1'b0;
  end
  else 
  begin
    wb_addr <= next_wb_addr;
    wb_data_o <= next_wb_data_o;
    wb_stb <= next_wb_stb;
    wb_we <= next_wb_we;
    wbBusReq <= next_wbBusReq;
    initComplete <= next_initComplete;
    memAddr <= next_memAddr;
    memRdEn <= next_memRdEn;
    cnt <= next_cnt;
    txDataSize <= next_txDataSize;
    txDataIndex <= next_txDataIndex;
    transDone <= next_transDone;
    epStatus <= next_epStatus;
    rxDataSize <= next_rxDataSize;
    epTransType <= next_epTransType;
    bm_req_dir <= next_bm_req_dir;
    bm_req_type <= next_bm_req_type;
    bm_req_recp <= next_bm_req_recp;
    bRequest <= next_bRequest;
    wValue <= next_wValue;
    wIndex <= next_wIndex;
    wLength <= next_wLength;
    txPacketRemSize <= next_txPacketRemSize;
    USBAddress <= next_USBAddress;
    updateUSBAddress <= next_updateUSBAddress;
    updateSetLineCoding <= next_updateSetLineCoding;
    updateGetLineCoding <= next_updateGetLineCoding;
    dataSeq <= next_dataSeq;
    stateDefault <= next_stateDefault;
    stateAddress <= next_stateAddress;
    stateConfigured <= next_stateConfigured;
  end
end

integer i;
wire w_wr_SetLineCodingBuf;
assign w_wr_SetLineCodingBuf = (CurrState_EP0St ==  P_OUT_GET_SET_LINE_CODING_DAT) & wb_ack & (rxDataSize != 16'h0);

wire [2:0] w_adr_SetLineCodingBuf;
assign w_adr_SetLineCodingBuf = (rxDataSize[2:0] == 3'd0) ? 3'd7 :
				(rxDataSize[2:0] == 3'd1) ? 3'd6 :
				(rxDataSize[2:0] == 3'd2) ? 3'd5 :
				(rxDataSize[2:0] == 3'd3) ? 3'd4 :
				(rxDataSize[2:0] == 3'd4) ? 3'd3 :
				(rxDataSize[2:0] == 3'd5) ? 3'd2 :
				(rxDataSize[2:0] == 3'd6) ? 3'd1 : 3'd0;
   
always @ (posedge clk) begin
  if (rst == 1'b1) begin
    // initial value
    SetLineCodingBuf[0] <= 8'h80;
    SetLineCodingBuf[1] <= 8'h25;
    SetLineCodingBuf[2] <= 8'h00;
    SetLineCodingBuf[3] <= 8'h00;
    SetLineCodingBuf[4] <= 8'h00;
    SetLineCodingBuf[5] <= 8'h00;
    SetLineCodingBuf[6] <= 8'h08;
    SetLineCodingBuf_1z <= 8'h00;
  end else begin
    SetLineCodingBuf_1z <=  w_memData_mux;
    if (w_wr_SetLineCodingBuf) SetLineCodingBuf[w_adr_SetLineCodingBuf] <= wb_data_i;
  end
end

   
endmodule
