//=======================================================================
//                         DE0 USB CDC Target
//
// File:
//   usbCdcROM.v
//
// Abstract:
//   CDC descriptor table
//
//  Created:
//    27 April 2015
//======================================================================
//
// Copyright (c) 2015, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id: 153e3ebf954dc5713cd38a0d78f3b669aa04c6c0 $

`include "usbCdcDevice_define.v"

module usbCdcROM (
  input clk,
  input [7:0] addr,
  output reg [7:0] data
);
/*
input clk;
input [7:0] addr;
output [7:0] data;
reg [7:0] data;
*/
always @(posedge clk) begin
  case (addr)
// ====================================
// =====    DEVICE Descriptor     =====
// ====================================

    8'h00: data <= 8'h12;  //BYTE bLength
    8'h01: data <= 8'h01;  //BYTE bDescriptorType
    8'h02: data <= 8'h10;  //WORD (Lo) bcdUSB version supported
    8'h03: data <= 8'h01;  //WORD (Hi) bcdUSB version supported
    8'h04: data <= 8'h02;  //BYTE bDeviceClass
    8'h05: data <= 8'h00;  //BYTE bDeviceSubClass
    8'h06: data <= 8'h08;  //BYTE bDeviceProtocol
    8'h07: data <= `MAX_RESP_SIZE;  //BYTE bMaxPacketSize 
    8'h08: data <= 8'hff;  //WORD (Lo) idVendor
    8'h09: data <= 8'h26;  //WORD (Hi) idVendor
    8'h0a: data <= 8'h12;  //WORD (Lo) idProduct; For Philips Hub mouse
    8'h0b: data <= 8'h49;  //WORD (Hi) idProduct; For Philips Hub mouse
    8'h0c: data <= 8'h00;  //WORD (Lo) bcdDevice
    8'h0d: data <= 8'h01;  //WORD (Hi) bcdDevice
    8'h0e: data <= 8'h01;  //BYTE iManufacturer
    8'h0f: data <= 8'h02;  //BYTE iProduct
    8'h10: data <= 8'h03;  //BYTE iSerialNumber
    8'h11: data <= 8'h01;  //BYTE bNumConfigurations

 
// ====================================
// ===== Configuration Descriptor =====
// ====================================
    8'h12: data <= 8'h09;  //BYTE bLength (Configuration descriptor)
    8'h13: data <= 8'h02;  //BYTE bDescriptorType //Assigned by USB
    8'h14: data <= 8'h43;  //WORD (Lo) wTotalLength
    8'h15: data <= 8'h00;  //WORD (Hi) wTotalLength
    8'h16: data <= 8'h02;  //BYTE bNumInterfaces
    8'h17: data <= 8'h01;  //BYTE bConfigurationValue
    8'h18: data <= 8'h00;  //BYTE iConfiguration
    8'h19: data <= 8'hc0;  //BYTE bmAttributes, Bus powered and remote wakeup
    8'h1a: data <= 8'hfa;  //BYTE MaxPower, 100mA
 
// ====================================
// =====   Interface Descriptor 0 =====
// ====================================
    8'h1b: data <= 8'h09;  //BYTE bLength (Interface descriptor)
    8'h1c: data <= 8'h04;  //BYTE bDescriptionType
    8'h1d: data <= 8'h00;  //BYTE bInterfaceNumber
    8'h1e: data <= 8'h00;  //BYTE bAlternateSetting
    8'h1f: data <= 8'h01;  //BYTE bNumEndpoints
    8'h20: data <= 8'h02;  //BYTE bInterfaceClass
    8'h21: data <= 8'h02;  //BYTE bInterfaceSubClass
    8'h22: data <= 8'h01;  //BYTE bInterfaceProtocol
    8'h23: data <= 8'h00;  //BYTE iInterface
    // Header Functional Descriptor
    8'h24: data <= 8'h05;  // bFunctionLength
    8'h25: data <= 8'h24;  // bDescriptorType
    8'h26: data <= 8'h00;  // bDescriptorSubType
    8'h27: data <= 8'h10;
    8'h28: data <= 8'h01;  // bcdCDC (CDC version 1.1)
    // Abstract Control Management Functional Descriptor
    8'h29: data <= 8'h04;  // bFunctionLength
    8'h2a: data <= 8'h24;  // bDescriptorType
    8'h2b: data <= 8'h02;  // bDescriptorSubType
    8'h2c: data <= 8'h02;  // bmCapabilities (Set/Get LineCoding support)
    // Union Interface Function Descriptor
    8'h2d: data <= 8'h05;  // bFunctionLength
    8'h2e: data <= 8'h24;  // bDescriptorType
    8'h2f: data <= 8'h06;  // bDescriptorSubType
    8'h30: data <= 8'h00;  // bMasterInterface0 (Interface#0 is control class)
    8'h31: data <= 8'h01;  // bSlaveInterface0 (Interface#1 is slave(Data class))
    // CallManagement Function Descriptor
    8'h32: data <= 8'h05;  // bFunctionLength
    8'h33: data <= 8'h24;  // bDescriptorType
    8'h34: data <= 8'h01;  // bDescriptorSubType
    8'h35: data <= 8'h00;  // bCapability
    8'h36: data <= 8'h01;  // bDataInterface
    // Endpoint Descriptor 3
    8'h37: data <= 8'h07;  // bLength
    8'h38: data <= 8'h05;  // bDescriptorType
    8'h39: data <= 8'h83;  // bEndpointAddress IN,3
    8'h3a: data <= 8'h03;  // bmAttributes: Interrupt
    8'h3b: data <= 8'h08;
    8'h3c: data <= 8'h00;  // wMaxPacketSize
    8'h3d: data <= 8'h0a;  // bInterval: 10ms
// ====================================
// =====   Interface Descriptor 1 =====
// ====================================
    8'h3e: data <= 8'h09;  // bLength
    8'h3f: data <= 8'h04;  //  bDescriptorType
    8'h40: data <= 8'h01;  //  bInterfaceNumber
    8'h41: data <= 8'h00;  //  bAlernateSetting
    8'h42: data <= 8'h02;  //  bNumEndpoints
    8'h43: data <= 8'h0a;  // Data Interface Class :bInterfaceClass
    8'h44: data <= 8'h00;  // bInterfaceSubClass
    8'h45: data <= 8'h00;  // bInterfaceProtocol
    8'h46: data <= 8'h00;  // iInterface
    // Endpoint Descriptor 1
    8'h47: data <= 8'h07;  // bLength;
    8'h48: data <= 8'h05;  // bDescriptorType
    8'h49: data <= 8'h01;  // bEndpointAddress OUT,1
    8'h4a: data <= 8'h02;  // bmAttributes: Bulk
    8'h4b: data <= 8'h40;
    8'h4c: data <= 8'h00;  // wMaxPacketSize:64
    8'h4d: data <= 8'h00;  // bInterval: 10ms
  // Endpoint Descriptor 2
    8'h4e: data <= 8'h07;  // bLength
    8'h4f: data <= 8'h05;  // bDescriptorType
    8'h50: data <= 8'h82;  // bEndpointAddress IN,2
    8'h51: data <= 8'h02;  // bmAttributes: Bulk
    8'h52: data <= 8'h40;
    8'h53: data <= 8'h00;  // wMaxPacketSize:64
    8'h54: data <= 8'hcc;  // bInterval: 10ms
//    8'h54: data <= 8'h00;  // bInterval: 10ms

// ZERO_ZERO
    8'h6c: data <= 8'h00; 
    8'h6d: data <= 8'h00; 
// ONE_ZERO
    8'h6e: data <= 8'h01; 
    8'h6f: data <= 8'h00; 
// Vendor data
    8'h70: data <= 8'h00; 
    8'h71: data <= 8'h00; 

// =============================================
// =====   Language ID Descriptor(String0) =====
// =============================================
    8'h80: data <= 8'h04;  // bLength
    8'h81: data <= 8'h03;  // bDescriptorType = String Desc
    8'h82: data <= 8'h09;  // wLangID (Lo) (Lang ID for English = 0x0409)
    8'h83: data <= 8'h04;  // wLangID (Hi) (Lang ID for English = 0x0409)

// ====================================
// =====   string 1 Descriptor  =====
// ====================================
    8'h90: data <= 8'd44;  	// bLength
    8'h91: data <= 8'h03;     // bDescriptorType = String Desc
    // Noting that text is always unicode, hence the 'padding'
    8'h92: data <= "O";  8'h93: data <= 8'h00;
    8'h94: data <= "p";  8'h95: data <= 8'h00;
    8'h96: data <= "e";  8'h97: data <= 8'h00;
    8'h98: data <= "n";  8'h99: data <= 8'h00;
    8'h9a: data <= "C";  8'h9b: data <= 8'h00;
    8'h9c: data <= "o";  8'h9d: data <= 8'h00;
    8'h9e: data <= "r";  8'h9f: data <= 8'h00;
    8'ha0: data <= "e";  8'ha1: data <= 8'h00;
    8'ha2: data <= "s";  8'ha3: data <= 8'h00;
    8'ha4: data <= " ";  8'ha5: data <= 8'h00;
    8'ha6: data <= "U";  8'ha7: data <= 8'h00;
    8'ha8: data <= "S";  8'ha9: data <= 8'h00;
    8'haa: data <= "B";  8'hab: data <= 8'h00;
    8'hac: data <= " ";  8'had: data <= 8'h00;
    8'hae: data <= "D";  8'haf: data <= 8'h00;
    8'hb0: data <= "e";  8'hb1: data <= 8'h00;
    8'hb2: data <= "v";  8'hb3: data <= 8'h00;
    8'hb4: data <= "i";  8'hb5: data <= 8'h00;
    8'hb6: data <= "c";  8'hb7: data <= 8'h00;
    8'hb8: data <= "e";  8'hb9: data <= 8'h00;
    8'hba: data <= ".";  8'hbb: data <= 8'h00;

// ====================================
// =====   string 2 Descriptor  =====
// ====================================
    8'hc0: data <= 8'd22;   // bLength
    8'hc1: data <= 8'h03;   // bDescriptorType = String Desc
    8'hc2: data <= "U";  8'hc3: data <= 8'h00;
    8'hc4: data <= "S";  8'hc5: data <= 8'h00;
    8'hc6: data <= "B";  8'hc7: data <= 8'h00;
    8'hc8: data <= " ";  8'hc9: data <= 8'h00;
    8'hca: data <= "S";  8'hcb: data <= 8'h00;
    8'hcc: data <= "e";  8'hcd: data <= 8'h00;
    8'hce: data <= "r";  8'hcf: data <= 8'h00;
    8'hd0: data <= "i";  8'hd1: data <= 8'h00;
    8'hd2: data <= "a";  8'hd3: data <= 8'h00;
    8'hd4: data <= "l";  8'hd5: data <= 8'h00;

// ====================================
// =====   string 3 Descriptor  =====
// ====================================
    8'he0: data <= 8'd20;   // bLength
    8'he1: data <= 8'h03;   // bDescriptorType = String Desc
    8'he2: data <= "1";  8'he3: data <= 8'h00;
    8'he4: data <= "2";  8'he5: data <= 8'h00;
    8'he6: data <= "3";  8'he7: data <= 8'h00;
    8'he8: data <= "4";  8'he9: data <= 8'h00;
    8'hea: data <= " ";  8'heb: data <= 8'h00;
    8'hec: data <= "A";  8'hed: data <= 8'h00;
    8'hee: data <= "B";  8'hef: data <= 8'h00;
    8'hf0: data <= "C";  8'hf1: data <= 8'h00;
    8'hf2: data <= "D";  8'hf3: data <= 8'h00;

    default: data <= 8'h00;
  endcase
end

endmodule


 
