//=======================================================================
//                         DE0 USB CDC Target
//
// File:
//   wbArbCdc.v
//
// Abstract:
//   Wishbone bus arbiter
//
//  Created:
//    27 April 2015
//======================================================================
//
// Copyright (c) 2015, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id: ec2195dfc08f8b26d7934064d0dc1de6cd94bea8 $

module wbArbCdc (
  clk,
  rst,

  addr0_i,
  data0_i,
  stb0_i,
  we0_i,
  req0,
  gnt0,

  addr1_i,
  data1_i,
  stb1_i,
  we1_i,
  req1,
  gnt1,

  addr2_i,
  data2_i,
  stb2_i,
  we2_i,
  req2,
  gnt2,


  addr_o,
  data_o,
  stb_o,
  we_o
);

input clk;
input rst;

input [7:0] addr0_i;
input [7:0] data0_i;
input stb0_i;
input we0_i;
input req0;
output gnt0;
reg gnt0;

input [7:0] addr1_i;
input [7:0] data1_i;
input stb1_i;
input we1_i;
input req1;
output gnt1;
reg gnt1;

input [7:0] addr2_i;
input [7:0] data2_i;
input stb2_i;
input we2_i;
input req2;
output gnt2;
reg gnt2;


output [7:0] addr_o;
reg [7:0] addr_o;
output [7:0] data_o;
reg [7:0] data_o;
output stb_o;
reg stb_o;
output we_o;
reg we_o;

//local wires and regs
reg [1:0] muxSel;
reg [2:0] arbSt;

localparam P_REQ_0 = 3'd0;
localparam P_REQ_1 = 3'd1;
localparam P_REQ_2 = 3'd2;
localparam P_GNT_0 = 3'd3;
localparam P_GNT_1 = 3'd4;
localparam P_GNT_2 = 3'd5;

//arb
always @(posedge clk) begin
  if (rst == 1'b1) begin
    gnt0 <= 1'b0;
    gnt1 <= 1'b0;
    gnt2 <= 1'b0;
    muxSel <= 2'b00;
    arbSt <= P_REQ_0;
  end
  else begin
    case (arbSt)
      P_REQ_0: begin
        if (req0 == 1'b1)
          arbSt <= P_GNT_0;
        else
          arbSt <= P_REQ_1;
      end
      P_REQ_1: begin
        if (req1 == 1'b1)
          arbSt <= P_GNT_1;
        else
          arbSt <= P_REQ_2;
      end
      P_REQ_2: begin
        if (req2 == 1'b1)
          arbSt <= P_GNT_2;
        else
          arbSt <= P_REQ_0;
      end
      P_GNT_0: begin
        gnt0 <= 1'b1;
        muxSel <= 2'b00;
        if (req0 == 1'b0) begin
          arbSt <= P_REQ_1;
          gnt0 <= 1'b0;
        end
      end
      P_GNT_1: begin
        gnt1 <= 1'b1;
        muxSel <= 2'b01;
        if (req1 == 1'b0) begin
          arbSt <= P_REQ_2;
          gnt1 <= 1'b0;
        end
      end
      P_GNT_2: begin
        gnt2 <= 1'b1;
        muxSel <= 2'b10;
        if (req2 == 1'b0) begin
          arbSt <= P_REQ_0;
          gnt2 <= 1'b0;
        end
      end
    endcase
  end
end


//mux
always @* begin
  case (muxSel)
    2'b00: begin
      addr_o = addr0_i;
      data_o = data0_i;
      stb_o = stb0_i;
      we_o = we0_i;
    end
    2'b01: begin
      addr_o = addr1_i;
      data_o = data1_i;
      stb_o = stb1_i;
      we_o = we1_i;
    end
    2'b10: begin
      addr_o = addr2_i;
      data_o = data2_i;
      stb_o = stb2_i;
      we_o = we2_i;
    end
    default: begin
      addr_o = addr0_i;
      data_o = data0_i;
      stb_o = stb0_i;
      we_o = we0_i;
    end
  endcase
end


endmodule

