//=======================================================================
//                         DE0 USB CDC Target
//
// File:
//   usbslave_cdc_top.v
//
// Abstract:
//   USB CDC Target full-logic version top
//
//  Created:
//    27 April 2015
//======================================================================
//
// Copyright (c) 2015, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id: 4ad214ba46fb5f37f1c510897b94c859695e3378 $

module usbslave_cdc_top (
// system
input CLK,
input RST,
// USB
inout USB_DP,
inout USB_DN,
// MISC
output [9:0] LEDG,
output [7:0] HEX0,
// FLASH
output DCLK,
output SCE,
output SDO,
input DATA
);

wire clk_c1;
wire clk_c2;
   
wire w_usbSlaveVP_in;
wire w_usbSlaveVM_in;
wire w_usbSlaveVP_out;
wire w_usbSlaveVM_out;
wire w_usbSlaveOE_n;
   
assign USB_DP = (w_usbSlaveOE_n) ? 1'hz : w_usbSlaveVP_out;
assign USB_DN = (w_usbSlaveOE_n) ? 1'hz : w_usbSlaveVM_out;
assign w_usbSlaveVP_in = USB_DP;
assign w_usbSlaveVM_in = USB_DN;

assign LEDG[9:3] = 7'd0;

usbslave_system u0 (
  .clk_clk(CLK),
  .reset_reset_n(RST),
  .altpll_0_areset_conduit_export(),
  .altpll_0_locked_conduit_export(),
  .altpll_0_phasedone_conduit_export(),
  .altpll_0_c2_clk(clk_c2),
  .epcs_flash_controller_0_external_dclk  (DCLK),
  .epcs_flash_controller_0_external_sce   (SCE),
  .epcs_flash_controller_0_external_sdo   (SDO),
  .epcs_flash_controller_0_external_data0 (DATA) 
);

usbCdcDevice u_usbCdcDevice (
  .clk(clk_c2),
  .rst(~RST),
  .usbSlaveVP_in(w_usbSlaveVP_in),
  .usbSlaveVM_in(w_usbSlaveVM_in),
  .usbSlaveVP_out(w_usbSlaveVP_out),
  .usbSlaveVM_out(w_usbSlaveVM_out),
  .usbSlaveOE_n(w_usbSlaveOE_n),
  .stateDefault(LEDG[0]),
  .stateAddress(LEDG[1]),
  .stateConfigured(LEDG[2]),
`ifdef USE_7SEG
  .disp7Seg(HEX0),
`endif
  .USBFullSpeed(),
  .usbDPlusPullup(),
  .usbDMinusPullup(),
  .vBusDetect('b1)
);

endmodule